import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { BankRatesService } from '../services/bank-rates.service';
import { getCacheData } from '../utils/helper';
import { AgentWiseRatesService } from '../services/agent-wise-rates.service';
import { AgentWiseCommissionService } from '../services/agent-wise-commission.service';

@Component({
  selector: 'app-agent-wise-rates',
  templateUrl: './agent-wise-rates.component.html',
  styleUrls: ['./agent-wise-rates.component.css'],
})
export class AgentWiseRatesComponent {
  @ViewChild('manageAgentWiseRates') manageAgenctWiseRatesModal;
  bsModalRef: BsModalRef;

  type: any = 'add';

  receivingCountries: any = [];
  selectedReceivingCountry: any = 'All';
  receivingInsertCountries: any = [];

  agentWiseRates: any = [];

  receivingAgents: any = [];
  selectedReceivingAgent: any = 'All';
  receivingInsertAgents: any = [];

  sendingCountries: any = [];
  selectedSendingCountry: any = 'All';
  sendingInsertCountries: any = [];

  sendingAgents: any = [];
  selectedSendingAgent: any = 'All';
  sendingInsertAgents: any = [];

  sendingCurrencies: any = [];
  receivingCurrencies: any = [];
  modeOfPayments: any = [];
  settlementCurrencies: any = [];

  manageAgencyRatesForm = new FormGroup({
    FromCountryID: new FormControl('', Validators.required),
    FromCurrencyID: new FormControl('', Validators.required),
    FromAgentID: new FormControl('', Validators.required),
    ToCountryID: new FormControl('', Validators.required),
    ToCurrencyID: new FormControl('', Validators.required),
    ToAgentID: new FormControl('', Validators.required),
    ModeOfPaymentID: new FormControl('', Validators.required),
    SettlementCurrency: new FormControl('', Validators.required),
    Bid_Rate: new FormControl('', Validators.required),
    Offer_Rate: new FormControl('', Validators.required),
    Revalution_Rate: new FormControl('', Validators.required),
    Settelment_Rate: new FormControl('', Validators.required),
  });

  constructor(
    private bankService: BankRatesService,
    private agentWiseService: AgentWiseRatesService,
    private agentWiseCommissionService: AgentWiseCommissionService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSendingCountry();
    this.getSendingAgent();
    this.getReceivingCountry();
    this.getReceivingAgent();
    this.getAllAgentWiseRates();
  }

  getAllAgentWiseRates(
    sendingCountryId: any = 0,
    sendingAgentId: any = 0,
    receivingCountryId: any = 0,
    receivingAgentId: any = 0
  ) {
    this.agentWiseService
      .getAllAgentWiseRates(
        getCacheData('admin_token'),
        getCacheData('username'),
        sendingCountryId,
        sendingAgentId,
        receivingCountryId,
        receivingAgentId
      )
      .subscribe({
        next: (res: any) => {
          this.agentWiseRates = res;
        },
        error: (err) => {
          this.agentWiseRates = [];
        },
        complete: () => {},
      });
  }

  getReceivingCountry(userId: any = 0) {
    this.bankService
      .getReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        userId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCountries = res;
          this.receivingInsertCountries = [...res];
        },
        error: (err) => {
          this.receivingCountries = [];
        },
        complete: () => {},
      });
  }

  getReceivingAgent(countryId: any = 0) {
    this.bankService
      .getReceivingAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingAgents = res;
          this.receivingInsertAgents = [...res];
        },
        error: (err) => {
          this.receivingAgents = [];
        },
        complete: () => {},
      });
  }

  getSendingAgent(type: any = 0, sendingcountryId: any = 0) {
    this.agentWiseService
      .getSendingAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        type,
        sendingcountryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendingAgents = res;
          this.sendingInsertAgents = [...res];
        },
        error: (err) => {
          this.sendingAgents = [];
        },
        complete: () => {},
      });
  }

  getSendingCountry(userId: any = 0) {
    this.agentWiseService
      .getSendingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        userId
      )
      .subscribe({
        next: (res: any) => {
          this.sendingCountries = res;
          this.sendingInsertCountries = [...res];
        },
        error: (err) => {
          this.sendingCountries = [];
        },
        complete: () => {},
      });
  }

  onSendingInsertCountryChange(countryId) {
    if (countryId) {
      this.getSendingAgent(0, countryId);
      this.getSendCurrencyInsert(countryId);
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openAgencyRatesModal(type: any) {
    if (type == 'add') {
      this.manageAgencyRatesForm.reset({
        FromCountryID: 'None',
      });
    }
    this.type = type;
    this.openModal(this.manageAgenctWiseRatesModal);
  }

  get f() {
    return this.manageAgencyRatesForm.value;
  }

  addAgencyWiseRates() {
    if (!this.manageAgencyRatesForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.FromCountryID == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    let payload;
    payload = {
      FromCountryID: this.f.FromCountryID,
      FromCurrencyID: this.f.FromCurrencyID,
      FromAgentID: this.f.FromAgentID,
      ToCountryID: this.f.ToCountryID,
      ToCurrencyID: this.f.ToCurrencyID,
      ToAgentID: this.f.ToAgentID,
      ModeOfPaymentID: this.f.ModeOfPaymentID,
      SettlementCurrency: this.f.SettlementCurrency,
      Bid_Rate: this.f.Bid_Rate,
      Offer_Rate: this.f.Offer_Rate,
      Revalution_Rate: this.f.Revalution_Rate,
      Settelment_Rate: this.f.Settelment_Rate,
      EnteredBy: 'none',
    };

    if (this.type === 'edit') {
      payload = [
        {
          SettlementCurrency: this.f.SettlementCurrency,
          // Currency_ID: this.f.Currency_ID,
          Bid_Rate: Number(this.f.Bid_Rate),
          Offer_Rate: Number(this.f.Offer_Rate),
          Revalution_Rate: Number(this.f.Revalution_Rate),
          Settelment_Rate: Number(this.f.Settelment_Rate),
          // ExecutionTime: Number(this.f.ExecutionTime),
          // AutoRateflag: this.f.AutoRateflag,
          DtEntered: new Date(),
          TmEntered: new Date(),
          EnteredBy: 'none',
          Message: 'none',
        },
      ];
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addAgentWiseExchangeRates'
        : 'updateAgentWiseExchangeRates';

    this.agentWiseService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
      },
      error: (err) => {
        alert(
          'Bank Agent Wise Currency Exchange Rate information added successfully.'
        );
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  onReceivingCountryChange() {
    if (this.selectedReceivingCountry != 'All') {
      this.getReceivingAgent(this.selectedReceivingCountry);
    }
  }

  closeModal() {
    this.bsModalRef.hide();
  }

  // editAgencyRates(agencyRates: any) {
  //   const settlementCurrencyId = this.settlementCurrencies.find(
  //     (p) => p.CurrencyShortName == agencyRates.SettlementCurrency
  //   )?.Currency_Id;
  //   const currencyId = this.currencies.find(
  //     (p) => p.CurrencyShortName == agencyRates.CurrencyShortName
  //   )?.Id;
  //   this.manageAgencyRatesForm.patchValue({
  //     SettlementCurrency: settlementCurrencyId,
  //     Currency_ID: currencyId,
  //     Bid_Rate: agencyRates?.BidRateUSD,
  //     Offer_Rate: agencyRates?.OfferRateUSD,
  //     Revalution_Rate: agencyRates?.RevaluationRateUSD,
  //     Settelment_Rate: agencyRates?.SettelmentRateUSD,
  //     AutoRateflag: agencyRates?.AutoRateFlag,
  //   });
  //   this.openAgencyRatesModal('edit');
  // }

  searchBankRates() {
    let sendingCountryId: number | null = 0;
    let sendingAgentId: number | null = 0;
    let receivingCountryId: number | null = 0;
    let receivingAgentId: number | null = 0;
    if (this.selectedSendingCountry !== 'All') {
      sendingCountryId = this.selectedSendingCountry;
    }

    if (this.selectedSendingAgent !== 'All') {
      sendingAgentId = this.selectedSendingAgent;
    }

    if (this.selectedReceivingCountry !== 'All') {
      receivingCountryId = this.selectedReceivingCountry;
    }
    if (this.selectedReceivingAgent !== 'All') {
      receivingAgentId = this.selectedReceivingAgent;
    }
    this.getAllAgentWiseRates(
      sendingCountryId,
      sendingAgentId,
      receivingCountryId,
      receivingAgentId
    );
  }

  onSendingCountryChange() {
    if (this.selectedSendingCountry != 'All') {
      this.getSendingAgent(0, this.selectedSendingCountry);
    }
  }

  onReceiveCountryInsertChange(countryId) {
    if (countryId) {
      this.getReceivingCurrencyInsert(countryId);
    }
  }

  onReceiveCurrencyInsertChange(countryId) {
    if (this.f.ToCountryID) {
      this.getReceivingAgent(this.f.ToCountryID);
    }
  }

  getSendCurrencyInsert(countryId: any = 0) {
    this.agentWiseCommissionService
      .getSendCurrencyInsert(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendingCurrencies = res;
        },
        error: (err) => {
          this.sendingCurrencies = [];
        },
        complete: () => {},
      });
  }

  getReceivingCurrencyInsert(countryId: any = 0) {
    this.bankService
      .getReceivingCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCurrencies = res;
        },
        error: (err) => {
          this.receivingCurrencies = [];
        },
        complete: () => {},
      });
  }

  onReceiveAgentInsertChange(agentId) {
    if (this.f.ToCountryID && agentId) {
      this.getModeOfPayment(this.f.ToCountryID, agentId);
      this.getSettlementCurrency(agentId);
    }
  }

  getModeOfPayment(targetCountryId: any = 0, receiveAgentId: any = 0) {
    this.agentWiseService
      .getModeOfPayment(targetCountryId, receiveAgentId)
      .subscribe({
        next: (res: any) => {
          this.modeOfPayments = res;
        },
        error: (err) => {
          this.modeOfPayments = [];
        },
        complete: () => {},
      });
  }

  getSettlementCurrency(agentId: any = 0) {
    this.agentWiseService.getSettlementCurrency(agentId).subscribe({
      next: (res: any) => {
        this.settlementCurrencies = res;
      },
      error: (err) => {
        this.settlementCurrencies = [];
      },
      complete: () => {},
    });
  }

  allowOneDecimal(event: Event): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
