import { Component, ElementRef, ViewChild } from '@angular/core';
import { AllSentTransactionsService } from '../services/all-sent-transactions.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-all-sent-transactions',
  templateUrl: './all-sent-transactions.component.html',
  styleUrls: ['./all-sent-transactions.component.css'],
})
export class AllSentTransactionsComponent {
  @ViewChild('transactionTable') transactionTable: ElementRef;

  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any = -1;
  agentBranchList: any = [];
  selectedAgentBranchList: any = 'All';

  startDate: any;
  endDate: any;

  allSentTransaction: any = [];
  agentName: any = '';
  totalPayInAmt: any = '';
  totalCommission: any = '';
  totalVat: any = '';
  selectedFileFormat: any = '1';

  constructor(private transactionSvc: AllSentTransactionsService) {}

  ngOnInit(): void {
    this.getCountryList();
  }

  getCountryList() {
    this.transactionSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
      },
      error: () => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 'All') {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.transactionSvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: () => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  onAgentChange(agentId: any) {
    if (agentId != 'All') {
      this.getAgentBranchList(agentId);
    }
  }

  getAgentBranchList(agentId: any = 0) {
    this.transactionSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
      },
      error: () => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  displayByDate() {
    if (!this.startDate && !this.endDate) {
      alert('Please Select From & To Date!');
      return;
    }

    if (!this.selectedCountryList) {
      alert('Please Select Country!');
      return;
    }

    let payload: any = {};

    payload = {
      FromDate: this.startDate,
      ToDate: this.endDate,
      CountryID: this.selectedCountryList,
      AgentID: this.selectedAgentList ? this.selectedAgentList : -1,
      AgentBranchID: this.selectedAgentBranchList
        ? this.selectedAgentBranchList
        : 'All',
      UserGroupType: 0,
      SessionAgent_ID: 0,
    };
    this.transactionSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        this.allSentTransaction = res;
        this.totalPayInAmt = res?.[0]?.PayInAmount;
        this.totalVat = res?.[0]?.Tot_VAT;
        this.totalCommission = res?.[0]?.Tot_Comm;
      },
      error: () => {
        this.allSentTransaction = [];
      },
      complete: () => {},
    });
  }

  get agent() {
    return (this.agentName = this.agentList.find(
      (a) => a.Agent_ID == this.selectedAgentList
    )?.Agent_Name);
  }

  get totalNoofTrxns(): number {
    return this.allSentTransaction.length;
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Agentwise_Transaction_Sent${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }
}
