import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import {
  FormGroup,
  FormControl,
  Validators,
  AbstractControl,
} from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { BankRatesService } from '../services/bank-rates.service';
import { AgencyService } from '../services/agency-rates.service';

@Component({
  selector: 'app-bank-rates',
  templateUrl: './bank-rates.component.html',
  styleUrls: ['./bank-rates.component.css'],
})
export class BankRatesComponent implements OnInit {
  @ViewChild('manageBankRates') manageBankRatesModal;
  bsModalRef: BsModalRef;

  type: any = 'add';

  receivingCountries: any = [];
  selectedReceivingCountry: any = 'All';

  bankRates: any = [];

  receivingAgents: any = [];
  selectedReceivingAgent: any = 'All';

  receivingCurrencies: any = [];
  agentAccountList: any = [];
  modeofPaymentList: any = [];
  settlementCurrencies: any = [];

  manageBankRatesForm = new FormGroup({
    AutoRateflag: new FormControl(false),
    Settelment_Rate: new FormControl('', Validators.required),
    Revalution_Rate: new FormControl('', Validators.required),
    Offer_Rate: new FormControl('', Validators.required),
    Bid_Rate: new FormControl('', Validators.required),
    ModeOfPayment_ID: new FormControl('', Validators.required),
    Agent_ID: new FormControl('', Validators.required),
    ToCurrency_Id: new FormControl('', Validators.required),
    ToCountry_ID: new FormControl('None', Validators.required),
    SettlementCurrency: new FormControl('', Validators.required),
    ID: new FormControl(),
  });

  constructor(
    private bankService: BankRatesService,
    private modalService: BsModalService,
    private agencyService: AgencyService
  ) {}

  ngOnInit(): void {
    this.getReceivingCountry();
    this.getAllBankRates();
    this.getSettlementCurrencies();
    this.getReceivingCurrency();
    this.manageBankRatesForm
      .get('Offer_Rate')
      .setValidators([Validators.required, this.offerRateValidator.bind(this)]);
    this.manageBankRatesForm
      .get('Settelment_Rate')
      .setValidators([
        Validators.required,
        this.settlementRateValidator.bind(this),
      ]);
  }

  offerRateValidator(
    control: AbstractControl
  ): { [key: string]: boolean } | null {
    const bidRate = this.manageBankRatesForm.get('Bid_Rate').value;
    const offerRate = control.value;

    return offerRate >= bidRate ? null : { offerRateInvalid: true };
  }

  settlementRateValidator(
    control: AbstractControl
  ): { [key: string]: boolean } | null {
    const bidRate = this.manageBankRatesForm.get('Bid_Rate').value;
    const settlementRate = control.value;

    return settlementRate >= bidRate ? null : { settlementRateInvalid: true };
  }

  getAllBankRates(countryId: any = 0, agentId: any = 0) {
    this.bankService
      .getAllBankRates(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        agentId
      )
      .subscribe({
        next: (res: any) => {
          this.bankRates = res;
          for (let index = 0; index < this.bankRates.length; index++) {
            const element = this.bankRates[index];
            if (element?.BidRate !== null && element?.BidRate !== undefined) {
              element.BidRate = this.formatWithSixDecimalPlaces(
                element.BidRate
              );
            }
            if (
              element?.OfferRate !== null &&
              element?.OfferRate !== undefined
            ) {
              element.OfferRate = this.formatWithSixDecimalPlaces(
                element.OfferRate
              );
            }
            if (
              element?.RevalutionRate !== null &&
              element?.RevalutionRate !== undefined
            ) {
              element.RevalutionRate = this.formatWithSixDecimalPlaces(
                element.RevalutionRate
              );
            }
            if (
              element?.SettelmentRate !== null &&
              element?.SettelmentRate !== undefined
            ) {
              element.SettelmentRate = this.formatWithSixDecimalPlaces(
                element.SettelmentRate
              );
            }
          }
        },
        error: (err) => {
          this.bankRates = [];
        },
        complete: () => {},
      });
  }

  formatWithSixDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure six decimal places
      return `${integerPart}.${fractionalPart.padEnd(6, '0').slice(0, 6)}`;
    } else {
      // If no decimal places, add six zeros
      return `${integerPart}.000000`;
    }
  }

  getReceivingCountry(userId: any = 0) {
    this.bankService
      .getReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        userId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCountries = res;
        },
        error: (err) => {
          this.receivingCountries = [];
        },
        complete: () => {},
      });
  }

  getReceivingAgent(countryId: any = 0) {
    this.bankService
      .getReceivingAgent(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingAgents = res;
        },
        error: (err) => {
          this.receivingAgents = [];
        },
        complete: () => {},
      });
  }

  getReceivingCurrency(countryId: any = 0) {
    this.bankService
      .getReceivingCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCurrencies = res;
        },
        error: (err) => {
          this.receivingCurrencies = [];
        },
        complete: () => {},
      });
  }

  getAgentAccountList(countryId: any = 0, currencyId: any = 0) {
    this.bankService
      .getAgentAccountList(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        currencyId
      )
      .subscribe({
        next: (res: any) => {
          this.agentAccountList = res;
        },
        error: (err) => {
          this.agentAccountList = [];
        },
        complete: () => {},
      });
  }

  getModeofPaymentList(agentId: any = 0) {
    this.bankService
      .getModeofPaymentList(
        getCacheData('admin_token'),
        getCacheData('username'),
        agentId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPaymentList = res;
        },
        error: (err) => {
          this.modeofPaymentList = [];
        },
        complete: () => {},
      });
  }

  getSettlementCurrencies() {
    this.agencyService
      .getSettlementCurrencies(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.settlementCurrencies = res;
        },
        error: (err) => {
          this.settlementCurrencies = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openBankRatesModal(type: any) {
    if (type == 'add') {
      this.manageBankRatesForm.reset({
        ToCountry_ID: 'None',
        ToCurrency_Id: 'None',
        Agent_ID: 'None',
        ModeOfPayment_ID: 'None',
      });
    } else {
      this.getReceivingCurrency();
      this.getAgentAccountList();
    }
    this.type = type;
    this.openModal(this.manageBankRatesModal);
  }

  get f() {
    return this.manageBankRatesForm.value;
  }

  addBankRates() {
    if (!this.manageBankRatesForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.ToCountry_ID == 'None') {
      alert('Please Select Receive Country!');
      return;
    }

    if (this.f.ToCurrency_Id == 'None') {
      alert('Please Select Receive Currency!');
      return;
    }

    if (this.f.Agent_ID == 'None') {
      alert('Please Select Agent Account!');
      return;
    }

    if (this.f.ModeOfPayment_ID == 'None') {
      alert('Please Select Mode of Payment!');
      return;
    }

    if (this.f.SettlementCurrency == '') {
      alert('Please Select Settlement Currency!');
      return;
    }

    if (this.f.Offer_Rate < this.f.Bid_Rate) {
      alert(
        'Offer Rate [USD] should be greater than or equal to Bid Rate [USD]!'
      );
      return;
    }

    if (this.f.Settelment_Rate < this.f.Bid_Rate) {
      alert(
        'Settlement Rate [USD] should be greater than or equal to Bid Rate [USD]!'
      );
      return;
    }

    let payload;
    payload = {
      SettlementCurrency: this.f.SettlementCurrency,
      ToCountry_ID: this.f.ToCountry_ID,
      ToCurrency_Id: this.f.ToCurrency_Id,
      Agent_ID: this.f.Agent_ID,
      ModeOfPayment_ID: this.f.ModeOfPayment_ID,
      Bid_Rate: Number(this.f.Bid_Rate),
      Offer_Rate: Number(this.f.Offer_Rate),
      Revalution_Rate: Number(this.f.Revalution_Rate),
      Settelment_Rate: Number(this.f.Settelment_Rate),
      EnteredBy: 'none',
      AutoRateflag: this.f.AutoRateflag,
    };

    if (this.type === 'edit') {
      payload = [
        {
          ID: this.f.ID,
          SettlementCurrency: this.f.SettlementCurrency,
          ToCountry_ID: this.f.ToCountry_ID,
          ToCurrency_Id: this.f.ToCurrency_Id,
          Agent_ID: this.f.Agent_ID,
          ModeOfPayment_ID: this.f.ModeOfPayment_ID,
          Bid_Rate: Number(this.f.Bid_Rate),
          Offer_Rate: Number(this.f.Offer_Rate),
          Revalution_Rate: Number(this.f.Revalution_Rate),
          Settelment_Rate: Number(this.f.Settelment_Rate),
          EnteredBy: 'none',
          AutoRateflag: this.f.AutoRateflag,
          Active: true,
        },
      ];
    }

    const serviceMethod =
      this.type === 'add' ? 'addBankRates' : 'updateBankRates';

    this.bankService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.getAllBankRates();
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  onReceivingCountryChange() {
    if (this.selectedReceivingCountry != 'All') {
      this.getReceivingAgent(this.selectedReceivingCountry);
    }
  }

  closeModal() {
    this.bsModalRef.hide();
  }

  editBankRates(bankRates: any) {
    let currencyId: any = 0;
    if (bankRates?.CountryID) {
      this.onReceiveCountryChange(bankRates?.CountryID);
    }
    if (this.receivingCurrencies?.length) {
      currencyId = this.receivingCurrencies.find(
        (p) => p.CurrencyShortName == bankRates.ReceiveCurrency
      )?.Currency_Id;
      if (currencyId) {
        this.onReceiveCurrencyChange(bankRates?.CountryID, currencyId);
      }
    }
    if (bankRates?.Agent_ID) {
      this.onAgentAccountChange(bankRates?.Agent_ID);
    }
    this.manageBankRatesForm.patchValue({
      ID: bankRates?.ID,
      Agent_ID: bankRates?.Agent_ID,
      ToCountry_ID: bankRates?.CountryID,
      ToCurrency_Id: currencyId,
      ModeOfPayment_ID: bankRates?.ModeOfPayment,
      Bid_Rate: bankRates?.BidRate,
      Offer_Rate: bankRates?.OfferRate,
      Revalution_Rate: bankRates?.RevalutionRate,
      Settelment_Rate: bankRates?.SettelmentRate,
      AutoRateflag: bankRates?.AutoRateFlag,
    });
    this.openBankRatesModal('edit');
  }

  searchBankRates() {
    let countryId: number | null = 0;
    let agentId: number | null = 0;

    if (this.selectedReceivingCountry !== 'All') {
      countryId = this.selectedReceivingCountry;
    }
    if (this.selectedReceivingAgent !== 'All') {
      agentId = this.selectedReceivingAgent;
    }
    this.getAllBankRates(countryId, agentId);
  }

  onReceiveCountryChange(countryId) {
    if (countryId != 'None') {
      this.getReceivingCurrency(countryId);
    }
  }

  onReceiveCurrencyChange(countryId, currencyId) {
    let countryID: any = 0;
    if (this.f.ToCountry_ID != 'None') {
      countryID = this.f.ToCountry_ID;
    } else {
      countryID = countryId;
    }
    if (currencyId != 'None') {
      this.getAgentAccountList(countryID, currencyId);
    }
  }

  onAgentAccountChange(agentId) {
    if (agentId != 'None') {
      this.getModeofPaymentList(agentId);
    }
  }

  isCheckOfferRate(bidRate: number, offerRate: number): boolean {
    return Number(offerRate) < Number(bidRate);
  }

  isCheckSettlementRate(bidRate: number, settlementRate: number): boolean {
    return Number(settlementRate) < Number(bidRate);
  }

  updateBankRates(): void {
    const updateBankRates = this.bankRates
      .filter((con) => con.isChecked)
      .map((con) => {
        const settlementCurrencyId = this.settlementCurrencies.find(
          (c) => c.CurrencyShortName === con.SettlementCCY
        )?.Currency_Id;
        const currencyId = this.receivingCurrencies.find(
          (c) => c.CurrencyShortName === con.ReceiveCurrency
        )?.Currency_Id;

        return {
          ID: con?.ID,
          SettlementCurrency: settlementCurrencyId,
          ToCountry_ID: con?.CountryID,
          ToCurrency_Id: currencyId,
          Agent_ID: con?.Agent_ID,
          ModeOfPayment_ID: this.f.ModeOfPayment_ID,
          Bid_Rate: Number(con?.BidRate),
          Offer_Rate: Number(con?.OfferRate),
          Revalution_Rate: Number(con?.RevalutionRate),
          Settelment_Rate: Number(con?.SettelmentRate),
          EnteredBy: 'none',
          AutoRateflag: con?.AutoRateFlag,
          Active: true,
        };
      });
    if (updateBankRates?.length) {
      this.bankService
        .updateBankRates(
          getCacheData('admin_token'),
          getCacheData('username'),
          updateBankRates
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
            this.getAllBankRates();
          },
          error: (err) => {
            this.modalService.hide();
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  allowOneDecimal(event: Event): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
