import { Component, TemplateRef, ViewChild, OnInit } from '@angular/core';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { ComplianceUploadDataService } from '../services/compliance-upload-data.service';

@Component({
  selector: 'app-compliance-upload-data',
  templateUrl: './compliance-upload-data.component.html',
  styleUrls: ['./compliance-upload-data.component.css']
})
export class ComplianceUploadDataComponent implements OnInit {
  bsModalRef: BsModalRef;
  @ViewChild('fileUploadModalTemplate') fileUploadModalTemplate: TemplateRef<any>;


  fileType: string = 'excel'; // Default file type
  complianceData: any[] = [];
  users: string[] = [];
  selectedUser: string = ''; 

  constructor(
    private modalService: BsModalService,
    private complianceService: ComplianceUploadDataService
  ) {}

  ngOnInit() {
    this.fetchData();
  }

  openFileUploadModal() {
    this.bsModalRef = this.modalService.show(this.fileUploadModalTemplate, { class: 'modal-lg' });
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  closeFileUploadModal() {
    this.bsModalRef.hide();
  }

  fetchData() {
    this.complianceService.getComplianceData()
        .subscribe((data: any[]) => {
            this.complianceData = data;
            this.extractUsers();
        }, error => {
            console.error(error);
        });
  }

  extractUsers() {
    this.users = Array.from(new Set(this.complianceData.map(item => item.PM_USER_NM)));
  }

  uploadFile() {
   
    const userName = ''; // Change this as needed
    const userId = 1; // Change this as needed

    this.complianceService.uploadFile(this.fileType, userName, userId)
      .subscribe(response => {
        console.log('File uploaded successfully:', response);
        // Do something with the response if needed
      }, error => {
        console.error('Error uploading file:', error);
        // Handle error
      });
  }
}