import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from './utils/helper';

@Injectable({
  providedIn: 'root',
})
export class ManageCountryService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getCountries(token: any, username): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/country/country-list`,
      { headers: headers }
    );
  }

  getCountry(token: any, username: any, countryId = 0): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/country/country?countryID=${countryId}`,
      { headers: headers }
    );
  }

  getCurrency(token: any, username): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/country/add/currency-list
     `,
      {
        headers: headers,
      }
    );
  }

  getAllowedInternationalTransaction(token: any, username): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/country/add/international-transaction-type-list
     `,
      {
        headers: headers,
      }
    );
  }

  getAllowedDomesticTransaction(token: any, username): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/country/add/Domestic-transaction-type-list
     `,
      {
        headers: headers,
      }
    );
  }

  addCountry(token: any, username, payload: any): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.post(
      `${environment.apiUrl}/v1/api/master/country/add
     `,
      payload,
      {
        headers: headers,
      }
    );
  }

  updateCountry(token: any, username, payload: any): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/country/update
     `,
      payload,
      {
        headers: headers,
      }
    );
  }

  getCountryById(token: any, username, countryId: any = 0): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/country/countrybyid?countryID=${countryId}
     `,
      {
        headers: headers,
      }
    );
  }

  viewCountry(countryId: any = 0, moduleId: any = 0): Observable<any> {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/country/getviewcountry?CountryID=${countryId}&ModuleID=${moduleId}
     `,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  approveCountry(
    token: any,
    username,
    countryId: any,
    moduleId: any
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/country/approve?CountryID=${countryId}&ModuleID=${moduleId}
     `,
      '',
      {
        headers: headers,
      }
    );
  }
}
