import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { ManageCurrencyService } from '../services/manage-currency.service';
import { getCacheData } from '../utils/helper';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormGroup, FormControl, Validators } from '@angular/forms';

@Component({
  selector: 'app-manage-currency',
  templateUrl: './manage-currency.component.html',
  styleUrls: ['./manage-currency.component.css'],
})
export class ManageCurrencyComponent implements OnInit {
  @ViewChild('manageCurrency') manageCurrencyModal;
  bsModalRef: BsModalRef;

  currencies: any = [];

  type: any = 'add';

  allCurrencies: any = [];
  selectedCurrency = 'All';

  currencyTypes: any = [
    {
      id: 2,
      name: 'Sending',
    },
    {
      id: 3,
      name: 'Receiving',
    },
    {
      id: 4,
      name: 'Send/Receive',
    },
  ];

  manageCurrencyForm = new FormGroup({
    currency_id: new FormControl(),
    currency_name: new FormControl('', Validators.required),
    currency_short_name: new FormControl('', Validators.required),
    scale: new FormControl('', Validators.required),
    active: new FormControl(false),
    is_settlement_currency: new FormControl(false),
    entered_by: new FormControl(''),
    currency_type: new FormControl('Select Currency Type', Validators.required),
  });

  constructor(
    private currencyService: ManageCurrencyService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getAllCurrencies();
  }

  getAllCurrencies(currency_id: any = 0) {
    this.currencyService
      .getCurrencies(
        getCacheData('admin_token'),
        getCacheData('username'),
        currency_id
      )
      .subscribe({
        next: (res: any) => {
          this.currencies = res;
          if (!this.allCurrencies.length) {
            this.allCurrencies = [...res];
          }
        },
        error: (err) => {
          this.currencies = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openCurrencyModal(type: any) {
    this.type = type;
    if (this.type == 'add') {
      this.manageCurrencyForm.reset({
        currency_type: 'Select Currency Type',
      });
    }
    this.openModal(this.manageCurrencyModal);
  }

  get f() {
    return this.manageCurrencyForm.value;
  }

  addCurrency() {
    if (!this.manageCurrencyForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.currency_type == 'Select Currency Type') {
      alert('Please Select Currency Type!');
      return;
    }

    let payload;
    payload = {
      CurrencyName: this.f.currency_name,
      CurrencyShortName: this.f.currency_short_name,
      Scale: this.f.scale,
      Active: this.f.active,
      IsSettlementCurrency: this.f.is_settlement_currency,
      EnteredBy: this.f.entered_by || 'none',
      CurrencyType: this.f.currency_type,
    };

    const currency_type = this.currencyTypes.find(
      (c) => c.name == this.f.currency_type
    );

    if (currency_type) {
      payload = {
        ...payload,
        CurrencyType: currency_type?.id,
      };
    }

    if (this.type === 'edit') {
      payload = {
        ...payload,
        CurrencyId: this.f.currency_id,
      };
    }

    const serviceMethod =
      this.type === 'add' ? 'addCurrency' : 'updateCurrency';

    this.currencyService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllCurrencies();
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  editCurrency(currency: any) {
    this.manageCurrencyForm.patchValue({
      currency_id: currency?.ID,
      currency_name: currency?.CurrencyName,
      currency_short_name: currency?.CurrencyShortName,
      currency_type: currency?.CurrencyType,
      scale: currency?.Scale,
      active: currency?.Active,
      is_settlement_currency: currency?.IsSettlementCurrency,
      entered_by: currency?.EnteredBy,
    });
    this.openCurrencyModal('edit');
  }

  searchCurrency() {
    if (this.selectedCurrency != 'All') {
      this.getAllCurrencies(this.selectedCurrency);
    } else {
      this.getAllCurrencies();
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
