import { Component, TemplateRef, ViewChild } from '@angular/core';
import { SendingAgentWiseService } from '../services/sending-agent-wise.service';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';

@Component({
  selector: 'app-sending-agent-wise',
  templateUrl: './sending-agent-wise.component.html',
  styleUrls: ['./sending-agent-wise.component.css'],
})
export class SendingAgentWiseComponent {
  @ViewChild('sendingAgent') sendingAgentModal;
  bsModalRef: BsModalRef;

  countryList: any = [];
  selectedCountry: any = '-1';
  agentList: any = [];
  selectedAgent: any = '-1';
  sendCountryAddList: any = [];
  currencyList: any = [];
  settlementCurrencyList: any = [];

  sendingAgentWise: any = [];
  type: any = 'add';

  offerRateError: boolean = false;
  settlementRateError: boolean = false;

  sendingAgentForm = new FormGroup({
    ID: new FormControl(),
    FromCountry_ID: new FormControl('None', Validators.required),
    FromCurrency_Id: new FormControl('None', Validators.required),
    // Agent_ID: new FormControl('None', Validators.required),
    SettlementCurrency: new FormControl('None', Validators.required),
    Bid_Rate: new FormControl('', Validators.required),
    Offer_Rate: new FormControl('', Validators.required),
    Revalution_Rate: new FormControl('', Validators.required),
    Settelment_Rate: new FormControl('', Validators.required),
    IsActive: new FormControl(false),
  });

  constructor(
    private sendingAgentService: SendingAgentWiseService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSendCountryList();
    this.getSendCountryAddList();
  }

  getSendCountryList() {
    this.sendingAgentService.getSendCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.sendingAgentService.getSendAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  filter() {
    if (this.selectedCountry != '-1' && this.selectedAgent != '-1') {
      this.sendingAgentService.getFillGrid(this.selectedCountry, -1).subscribe({
        next: (res: any) => {
          this.sendingAgentWise = res;
        },
        error: (err) => {
          this.sendingAgentWise = [];
        },
        complete: () => {},
      });
    }
  }

  getSendCountryAddList() {
    this.sendingAgentService.getSendCountryAddList().subscribe({
      next: (res: any) => {
        this.sendCountryAddList = res;
      },
      error: (err) => {
        this.sendCountryAddList = [];
      },
      complete: () => {},
    });
  }

  onCountryAddChange(countryId: any) {
    if (countryId) {
      this.getSendCurrencyList(countryId);
      this.getSettlementCurrencyList(0);
    }
  }

  getSendCurrencyList(sendCountryId: any = 0) {
    this.sendingAgentService.getSendCurrencyList(sendCountryId).subscribe({
      next: (res: any) => {
        this.currencyList = res;
        this.sendingAgentForm.patchValue({
          FromCurrency_Id: 'None',
        });
      },
      error: (err) => {
        this.currencyList = [];
      },
      complete: () => {},
    });
  }

  getSettlementCurrencyList(agentId: any = 0) {
    this.sendingAgentService.getSettlementCurrencyList(agentId).subscribe({
      next: (res: any) => {
        this.settlementCurrencyList = res;
        this.sendingAgentForm.patchValue({
          SettlementCurrency: res?.[0]?.Currency_Id,
        });
      },
      error: (err) => {
        this.settlementCurrencyList = [];
      },
      complete: () => {},
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openSendingAgentWiseModal(type: any) {
    if (type == 'add') {
      this.sendingAgentForm.reset({
        FromCountry_ID: 'None',
      });
    }
    this.type = type;
    this.openModal(this.sendingAgentModal);
  }

  get f() {
    return this.sendingAgentForm.value;
  }

  allowOneDecimal(event: Event, rateType: string): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;

    this.checkOfferRate(rateType);
  }

  checkOfferRate(rateType: string): void {
    const bidRate = this.sendingAgentForm.get('Bid_Rate').value;
    const offerRate = this.sendingAgentForm.get('Offer_Rate').value;
    const settlementRate = this.sendingAgentForm.get('Settelment_Rate').value;

    switch (rateType) {
      case 'Offer_Rate':
        this.offerRateError = Number(offerRate) < Number(bidRate);
        break;

      case 'Settlement_Rate':
        this.settlementRateError = Number(settlementRate) < Number(bidRate);
        break;

      // Add more cases if needed for other rate types

      default:
        // Default case, if rateType doesn't match any of the above
        break;
    }
  }

  addSendingAgent() {
    if (!this.sendingAgentForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    let payload;
    payload = {
      FromCountry_ID: this.f.FromCountry_ID,
      FromCurrency_Id: this.f.FromCurrency_Id,
      Agent_ID: 0,
      SettlementCurrency: this.f.SettlementCurrency,
      Bid_Rate: this.f.Bid_Rate,
      Offer_Rate: this.f.Offer_Rate,
      Revalution_Rate: this.f.Revalution_Rate,
      Settelment_Rate: this.f.Settelment_Rate,
      UserID: 'string',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: Number(this.f.ID),
      };
    }

    const serviceMethod =
      this.type === 'add' ? 'addSendingAgentWise' : 'updateSendingAgentWise';

    this.sendingAgentService[serviceMethod](payload).subscribe({
      next: (res: any) => {
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  reset() {
    this.sendingAgentForm.reset({
      FromCountry_ID: 'None',
    });
    this.currencyList = [];
    this.settlementCurrencyList = [];
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
