import { Component, ElementRef, ViewChild } from '@angular/core';
import { TransactionAccountingDetailsService } from '../services/transaction-accounting-details.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-transaction-accounting-details',
  templateUrl: './transaction-accounting-details.component.html',
  styleUrls: ['./transaction-accounting-details.component.css'],
})
export class TransactionAccountingDetailsComponent {
  @ViewChild('accountingTable') accountingTable: ElementRef;

  countryList: any = [];
  selectedCountryList: any;
  agentList: any = [];
  selectedAgentList: any;
  fromDate: any = '';
  toDate: any = '';
  transactionNo: any = '';
  allAccountingDetails: any = [];
  selectedFileFormat: any = '1';

  constructor(
    private accountingDetailSvc: TransactionAccountingDetailsService
  ) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentListByCountry(0);
  }

  getCountryList() {
    this.accountingDetailSvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryList = res?.[0]?.CountryID;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != -1) {
      this.getAgentListByCountry(countryId);
    }
  }

  getAgentListByCountry(countryId: any = 0) {
    this.accountingDetailSvc.getAgentListByCountry(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        if (countryId == 0) {
          this.selectedAgentList = res?.[0]?.Agent_ID;
        }
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.fromDate && !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }

    let payload: any = {};

    payload = {
      FromDate: this.fromDate,
      ToDate: this.toDate,
      CountryID: -1,
      AgentID: -1,
      CustomerID: -1,
      TrxnNo: '',
    };

    this.accountingDetailSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        this.allAccountingDetails = res;
      },
      error: () => {
        this.allAccountingDetails = [];
      },
      complete: () => {},
    });
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Fill Transaction Number!');
      return;
    }

    let payload: any = {};

    payload = {
      TrxnNo: this.transactionNo,
    };

    this.accountingDetailSvc.getFillGrid(payload).subscribe({
      next: (res: any) => {
        this.allAccountingDetails = res;
      },
      error: () => {
        this.allAccountingDetails = [];
      },
      complete: () => {},
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.accountingTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Accounting_Details${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
