import { Component, ElementRef, ViewChild } from '@angular/core';
import { ViewDuplicateTransactionsService } from '../services/view-duplicate-transactions.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-view-duplicate-transactions',
  templateUrl: './view-duplicate-transactions.component.html',
  styleUrls: ['./view-duplicate-transactions.component.css'],
})
export class ViewDuplicateTransactionsComponent {
  @ViewChild('viewDuplicateTransactionTable')
  viewDuplicateTransactionTable: ElementRef;

  fromDate: any = '';
  toDate: any = '';
  transactionNumber: any = '';
  allDuplicateTransactions: any = [];
  selectedFileFormat: any = '1';

  constructor(
    private viewDuplicateTransactionSvc: ViewDuplicateTransactionsService
  ) {}

  ngOnInit(): void {}

  fetchRecords() {
    if (!this.fromDate && !this.toDate) {
      alert('Please Select From & To Date!');
      return;
    }

    let observable;

    if (!this.transactionNumber) {
      observable = this.viewDuplicateTransactionSvc.getAllRecord(
        this.fromDate,
        this.toDate
      );
    } else {
      observable = this.viewDuplicateTransactionSvc.getAllRecord(
        this.fromDate,
        this.toDate,
        this.transactionNumber
      );
    }

    observable.subscribe({
      next: (res: any) => {
        this.allDuplicateTransactions = res;
        if (this.allDuplicateTransactions?.length) {
          this.allDuplicateTransactions.forEach((item) => {
            if (typeof item === 'object' && item !== null) {
              item.Status = this.sanitizeHTML(item.Status);
            }
          });
        }
      },
      error: () => {
        this.allDuplicateTransactions = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    this.fetchRecords();
  }

  filterByTransactionNo() {
    if (!this.transactionNumber) {
      alert('Please Fill Transaction Number!');
      return;
    }

    this.fetchRecords();
  }

  sanitizeHTML(htmlString: string): string {
    const doc = new DOMParser().parseFromString(htmlString, 'text/html');
    return doc.body.textContent || '';
  }

  releaseTransaction(transactionNo, genuineTransactionNo) {
    if (transactionNo && genuineTransactionNo) {
      this.viewDuplicateTransactionSvc
        .releaseTransaction(transactionNo, genuineTransactionNo)
        .subscribe({
          next: (res: any) => {},
          error: (err) => {
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {},
        });
    }
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.viewDuplicateTransactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `View_Duplicate_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
